@@ -3,7 +3,7 @@
 ңУǺץ
 	 ꥨ ͥåȥңУǥ <ͥåȥط饤֥>
 	 Version 0.001, October 2000
-    $Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $
+    $Id: libnet.c,v 0.48 2004/01/06 09:49:10 naohki Exp $
 	 ------------------------------------------------------------
 	 Copyright (C) 2000,2001,2002,2003ǯ naoki kuwamori<naoki@kuwamori.com>
 
@@ -18,7 +18,7 @@
 
 	Laurier Net work RPG server<game network library>
 	Version 0.001, October 2000
-    $Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $
+    $Id: libnet.c,v 0.48 2004/01/06 09:49:10 naohki Exp $
 	Copyright (C) 2000,2001,2002,2003 Naoki Kuwamori <naoki@kuwamori.com>
 
 	This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 
 /etc/servicese game_rpgrpg_chatrpg_clientΥȥɬפ롣
 */
-static char cvs_id[] = "$Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $";
+static char cvs_id[] = "$Id: libnet.c,v 0.48 2004/01/06 09:49:10 naohki Exp $";
 
 
 /*
@@ -56,20 +56,92 @@ static char cvs_id[] = "$Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $";
 
 static int * rpg_FDS = NULL;
 static int * rpg_IDS = NULL;
+static int * rpg_FDTYPE = NULL;
 static int * rpg_IDSR = NULL;
 static struct sockaddr_in * rpg_UDPAD = NULL;
-
-
 struct pollfd *s_Poll = NULL;
-static int servfdT = 0;
-static int servfdU = 0;
-static int servfdUD = 0;
 static int rpg_fdnom = 0;
 struct rlimit fdlim;
 static int flag = 0;
+#undef RPGDEBUGLEVEL
 #define  RPGDEBUGLEVEL 1000
+#define  RPGDEBUGLEVEL2 2000
 
-void brakepipe(int data){
+/*local function place*/
+static int _serveropen(int type ,   char * service_name, int polltype );
+static int _rpg_tcp_client_open( char * hostname , char * service_name );
+static int read_packet (int FD     , int requestlen , int *read_len , char * buf , int bufsizei , int timeout);
+
+/**************************************************************************
+ *     
+ *                 ѥ»Хåؿ
+ *
+			:()
+				ѥפȤʤɤ˸ƤӽФ
+				Хåؿ
+				äˤʤˤ⤷ʤǵ
+
+			:
+				ʤ
+
+			:
+				ʤ
+*/
+static      void brakepipe(int data);
+
+/*
+ *             FD¸߳ǧؿ
+
+    ()
+        ǡǡ١ˤΣƣĤäƤ뤫ǧ
+        ơΥȥ꡼ֹ֤
+
+    :
+        int fd      I   Ĵ٤ƣ
+
+    :
+      0<
+          䥨ȥ꡼ֹ
+        0>
+          顼
+                LIBNETNOFDȥ꡼¸ߤʤ
+*/
+static  int test_fd(int fd );
+
+/**************************************************************************
+		FDϿؿ
+
+    :()
+        FDǡ١˿ȥä
+        Υȥֹ֤
+        Ρȥξˣ򥪡ץ󤷤
+        ƣĤ롣
+
+    
+        ʤ
+
+    ͡
+        Ƥƣĥǡ١ȥ
+*/
+
+static int getfdnewentry(void);
+
+/**************************************************************************
+		ǡ١ؿ
+    
+        ǡ١
+        ǽΰƤӽФ줿˽򤷤Ƥʹ
+        ˽򤹤Ȥ̤δؿƤӽФ
+
+    
+        ʤ
+
+*/
+static int clear_fddatabase(void);
+
+
+
+static void brakepipe(int data){
 	rpglog (RPGDEBUGLEVEL ,"libnet:get broken pipe\n");
 	/*ïΥʥ뤫Ϥ狼ʤҤȤޤfʤˤ⤷ʤ*/
 	/*Υ顼Ǥ롣*/
@@ -107,6 +179,7 @@ void rpg_endproc(int data){
 	if (rpg_FDS != NULL){free (rpg_FDS);rpg_FDS= NULL;}
 	if (s_Poll != NULL){free (s_Poll);s_Poll= NULL;}
 	if (rpg_IDS != NULL){free (rpg_IDS);rpg_IDS= NULL;}
+	if (rpg_FDTYPE != NULL){free (rpg_FDTYPE);rpg_FDTYPE= NULL;}
 	if (rpg_IDSR != NULL){free (rpg_IDSR);rpg_IDSR= NULL;}
 	if (rpg_UDPAD != NULL){free (rpg_UDPAD);rpg_UDPAD= NULL;}
 	
@@ -120,7 +193,8 @@ void rpg_endproc(int data){
 static int test_fd(int fd ){
 int cnt;
 
-	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
+	/*for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){*/
+	for (cnt = 0 ; cnt < rpg_fdnom ; cnt ++){
 		if (rpg_FDS[cnt] == fd){
 			return cnt;
 		}
@@ -138,6 +212,7 @@ int cnt;
 	for (cnt = 0 ; cnt < rpg_fdnom ; cnt ++ ){
 		if (rpg_FDS[cnt] == 0){
 			rpg_IDS[cnt] = (int)NULL;
+			rpg_FDTYPE[cnt] = (int)NULL;
 			rpg_IDSR[cnt] = (int)NULL;
     		rpg_UDPAD[cnt].sin_family = (int)NULL;
     		rpg_UDPAD[cnt].sin_port = (int)NULL;
@@ -147,9 +222,10 @@ int cnt;
 	}
 	/*Ǥ˳ƤȤ̵ȤϿΰ֤*/
 	rpg_IDS[rpg_fdnom]= 0;
+	rpg_FDTYPE[rpg_fdnom]=0;
 	rpg_IDSR[rpg_fdnom]=0;
-	rpg_UDPAD[cnt].sin_family = (int)NULL;
-	rpg_UDPAD[cnt].sin_port = (int)NULL;
+	rpg_UDPAD[rpg_fdnom].sin_family = (int)NULL;
+	rpg_UDPAD[rpg_fdnom].sin_port = (int)NULL;
 	rpg_fdnom++;
 	/*ƣĿƥΥߥåȤˤʤäϥ顼֤*/
 	if (rpg_fdnom >= fdlim.rlim_cur ){
@@ -164,12 +240,10 @@ int cnt;
 
 	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
 		if (rpg_FDS[cnt] == FD){
-			if (FD == servfdT ) servfdT = (int)NULL;
-			if (FD == servfdU ) servfdU = (int)NULL;
-			if (FD == servfdUD ) servfdUD = (int)NULL;
 			close (rpg_FDS[cnt]);
 			rpg_FDS[cnt] = (int)NULL;
 			rpg_IDS[cnt] = (int)NULL;
+			rpg_FDTYPE[cnt] = (int)NULL;
 			rpg_IDSR[cnt] = (int)NULL;
 			rpg_UDPAD[cnt].sin_family = (int)NULL;
 			rpg_UDPAD[cnt].sin_port = (int)NULL;
@@ -188,6 +262,7 @@ int cnt;
 			close (rpg_FDS[cnt]);
 			rpg_FDS[cnt] = ( int )NULL;
 			rpg_IDS[cnt] = ( int )NULL;
+			rpg_FDTYPE[cnt] = ( int )NULL;
 			rpg_IDSR[cnt] = ( int )NULL;
 			rpg_UDPAD[cnt].sin_family = (int)NULL;
 			rpg_UDPAD[cnt].sin_port = (int)NULL;
@@ -198,6 +273,7 @@ int cnt;
 	if (rpg_FDS != NULL){free (rpg_FDS);rpg_FDS= NULL;}
 	if (s_Poll != NULL){free (s_Poll);s_Poll= NULL;}
 	if (rpg_IDS != NULL){free (rpg_IDS);rpg_IDS= NULL;}
+	if (rpg_FDTYPE != NULL){free (rpg_FDTYPE);rpg_FDTYPE= NULL;}
 	if (rpg_IDSR != NULL){free (rpg_IDSR);rpg_IDSR= NULL;}
 	if (rpg_UDPAD != NULL){free (rpg_UDPAD);rpg_UDPAD= NULL;}
 	/*δؿͤʤΤͤϸʤ*/
@@ -217,6 +293,7 @@ int cnt;
 			close (rpg_FDS[cnt]);
 			rpg_FDS[cnt] = ( int )NULL;
 			rpg_IDS[cnt] = ( int )NULL;
+			rpg_FDTYPE[cnt] = ( int )NULL;
 			rpg_IDSR[cnt] = ( int )NULL;
 			rpg_UDPAD[cnt].sin_family = (int)NULL;
 			rpg_UDPAD[cnt].sin_port = (int)NULL;
@@ -262,6 +339,15 @@ int cnt;
 		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
 			rpg_IDS[cnt] = 0;
 		}
+		/*䥿פ*/
+		if (rpg_FDTYPE == NULL )rpg_FDTYPE = malloc( sizeof (int) * fdlim.rlim_cur );
+		if (rpg_FDTYPE == NULL){
+			rpglog ( RPGDEBUGLEVEL , "libnet:malloc error [%d]\n" , errno );
+			return LIBNETMALOCCERR;
+		}
+		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
+			rpg_FDTYPE[cnt] = 0;
+		}
 		/*ɤ߹ߤ褦Σֹ̤*/
 		if (rpg_IDSR == NULL )rpg_IDSR = malloc( sizeof (int) * fdlim.rlim_cur );
 		if (rpg_IDSR == NULL){
@@ -323,6 +409,7 @@ int cnt;
 /*ƣĥǡ١˥ݡ󥰤볰ƣĤϿ*/
 int rpg_addRawPollFD(int FD){
 int ent;
+	rpglog (RPGDEBUGLEVEL , "rpg_addRawPollFD:set RAW fd[%d]\n" ,FD );
 	if (test_fd(FD)!=LIBNETNOFD){
 		rpglog ( RPGDEBUGLEVEL , "libnet:The FD aredy seted\n");
 		return LIBNETFDAREDYSET;
@@ -331,13 +418,23 @@ int ent;
 
 	rpg_FDS[ent]=FD;
 	rpg_IDS[ent] = -1;
+	rpg_FDTYPE[ent] = POLLTYPE_RAW;
 	rpg_IDSR[ent] = -1;
 
 	return LIBNETOK ;
 }
 
+/*ФȤƥåȤ򥪡ץ󤹤*/
+int rpg_RawServerOpen(int type ,	char * service_name ){
+
+	return _serveropen (type,service_name,POLLTYPE_RAW);
+}
 /*ФȤƥåȤ򥪡ץ󤹤*/
 int rpg_serveropen(int type ,	char * service_name ){
+	return _serveropen (type,service_name,POLLTYPE_LAUREL);
+
+}	
+static int _serveropen(int type ,	char * service_name, int polltype ){
 
 struct servent *serv=NULL;
 struct sockaddr_in		saddrt;
@@ -354,9 +451,6 @@ int newfdent;
 	/*åȥפ˥ååȤξ礫ĥå̾Ȥ*/
 
 	if (type == AF_UNIX){
-		if (servfdU != 0){
-			return LIBNETOREDYSOPEN;
-		}
 		newfdent = getfdnewentry();
 		/*åȤ롣ΥåȤUNIXɥᥤǡȥ꡼෿åȡ*/
 		if ((rpg_FDS[newfdent] = socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
@@ -393,15 +487,17 @@ int newfdent;
 			perror("listen");
 			return LIBNETLISSTENERR;
 		}
-		servfdU = rpg_FDS[newfdent];
+		if (polltype == POLLTYPE_RAW){
+			rpg_IDS[newfdent] = -1;
+			rpg_FDTYPE[newfdent] = POLLTYPE_URAWSERV;
+			rpg_IDSR[newfdent] = -1;
+		}else{
+			rpg_FDTYPE[newfdent] = POLLTYPE_USERV;
+		}
 		return rpg_FDS[newfdent];
 
 	}else if (type == AF_INET){
 		int cnt;
-		if (servfdT != 0){
-
-			return LIBNETOREDYSOPEN;
-		}
 		newfdent = getfdnewentry();
 		/*TCPåȤξ*/
 		/* åȤ̾Ƥ*/
@@ -448,14 +544,17 @@ int newfdent;
 			perror("listen");
 			return LIBNETLISSTENERR;
 		}
-		servfdT = rpg_FDS[newfdent];
+		if (polltype == POLLTYPE_RAW){
+			rpg_IDS[newfdent] = -1;
+			rpg_FDTYPE[newfdent] = POLLTYPE_TRAWSERV;
+			rpg_IDSR[newfdent] = -1;
+		}else{
+			rpg_FDTYPE[newfdent] = POLLTYPE_TSERV;
+		}
 		rpglog(RPGDEBUGLEVEL , " libnet:success init \n");
 		return rpg_FDS[newfdent];
 	}else if(type == LIBNET_UDP) {
 		int cnt;
-		if (servfdUD!= 0){
-			return LIBNETOREDYSOPEN;
-		}
 		newfdent = getfdnewentry();
 		/*TCPåȤξ*/
 		/* åȤ̾Ƥ*/
@@ -495,8 +594,14 @@ int newfdent;
 			return LIBNETSOCBINDERR;
 		}
 
-		servfdUD = rpg_FDS[newfdent];
 		rpglog(RPGDEBUGLEVEL , " libnet:UDP sucsess init\n");
+		if (polltype == POLLTYPE_RAW){
+			rpg_IDS[newfdent] = -1;
+			rpg_FDTYPE[newfdent] = POLLTYPE_RAW;
+			rpg_IDSR[newfdent] = -1;
+		}else{
+			rpg_FDTYPE[newfdent] = POLLTYPE_LAUREL;
+		}
 		return rpg_FDS[newfdent];
 
 	}else{
@@ -509,11 +614,26 @@ int newfdent;
 
 /*饤ȤȤƣϩ򥪡ץ󤹤*/
 int rpg_tcp_client_open( char * hostname , char * service_name ){
+int FD;
+	FD = _rpg_tcp_client_open( hostname , service_name );
+	if (FD<=0){
+		return FD;
+	}
+	/*ңУǤȤƤ̿򤷤¾Υץफ³ǤʤȤΤ餻*/
+	if (write (FD , LIBNETASSEPT , strlen(LIBNETASSEPT))< 0){
+		close (FD);
+		rpg_FDS[test_fd(FD)] = 0;
+		return LIBNETCONECTERR;
+	}
+	return FD;
+
+}
+/*饤ȤȤƣϩ򥪡ץ󤹤*/
+static int _rpg_tcp_client_open( char * hostname , char * service_name ){
 	int FD;
 	struct servent *serv;
 	struct hostent *hp;
 	struct sockaddr_in	  addr;
-	int cnt;
 
 	if (flag==0){
 		clear_fddatabase();
@@ -539,20 +659,20 @@ int rpg_tcp_client_open( char * hostname , char * service_name ){
 	addr.sin_family = AF_INET;
 
 	/*ӥ̾ݡֹ롣*/
-	for (cnt=0;cnt<strlen(service_name);cnt++){
+	{
 		int testd;
-		testd = service_name[cnt];
+		/*ǽΣʸܤ򻲾Ȥƿʸ󤫤ѤƤ뤬ɤ*/
+		testd = service_name[0];
 		if (!isdigit(testd)){
 
 			/*ӥ̾ݡֹ롣*/
 			if ( (serv = getservbyname( service_name , "tcp" ) ) == NULL){
-				rpglog (RPGDEBUGLEVEL , "libnet:libnet:getservbyname %s error\n" , getservbyname);
+				rpglog (RPGDEBUGLEVEL , "libnet:libnet:getservbyname %s error\n" , getservbyname( service_name , "tcp"));
 				perror ("getservbyname");
 				return LIBNETGETPORTERR ;
 			}
 			addr.sin_port = serv->s_port;
 			rpglog (RPGDEBUGLEVEL ,"libnet:param port[%d]\n" , serv->s_port);
-			break;
 		}else{
 			addr.sin_port = htons((short)atoi(service_name));
 		}
@@ -566,11 +686,6 @@ int rpg_tcp_client_open( char * hostname , char * service_name ){
 		perror("connect");
 		return LIBNETCONECTERR;
 	}
-	/*ңУǤȤƤ̿򤷤¾Υץफ³ǤʤȤΤ餻*/
-	if (write (FD , LIBNETASSEPT , strlen(LIBNETASSEPT))< 0){
-		close (FD);
-		return LIBNETCONECTERR;
-	}
 	/*ץ󤷤ƣĤȤ*/
 	rpg_FDS[getfdnewentry()] = FD;
 	
@@ -580,12 +695,12 @@ int rpg_tcp_client_open( char * hostname , char * service_name ){
 int rpg_udp_client_open( char * hostname , char * service_name ){
 struct servent *serv;
 unsigned short port = 0;
-int cnt;
 
 /*ӥ̾ݡֹ롣*/
-	for (cnt=0;cnt<strlen(service_name);cnt++){
+	{
 		int testd;
-		testd = service_name[cnt];
+		/*ǽΣʸܤ򻲾Ȥƿʸ󤫤ѤƤ뤬ɤ*/
+		testd = service_name[0];
 		if (!isdigit(testd)){
 
 			/*ӥ̾ݡֹ롣*/
@@ -596,13 +711,12 @@ int cnt;
 			}
 			port = serv->s_port;
 			rpglog (RPGDEBUGLEVEL ,"param port[%d]\n" , serv->s_port);
-			break;
 		}else{
 			port = htons((short)atoi(service_name));
 		}
 	}
 
-return rpg_udp_client_open_port ( hostname , ntohs(port));
+	return rpg_udp_client_open_port ( hostname , ntohs(port));
 
 }
 int rpg_udp_client_open_port( char * hostname , int port ){
@@ -768,6 +882,7 @@ int cnt;
 int ret ;
 libnet_send_st send_st;	
 
+	rpglog(RPGDEBUGLEVEL+100 , "libnet:rpg_send_udp FD[%d]\n",FD);
 	if (FD == 0){
 		rpglog(RPGDEBUGLEVEL , "libnet: Need FD nomber\n");
 		return LIBNETNOFD;
@@ -941,7 +1056,7 @@ static int read_packet(int FD	 , int requestlen , int *read_len , char * buf , i
 	}
 	stat = LIBNETOK;
 	*read_len = 0;
-	rpglog(RPGDEBUGLEVEL , "libnet:main loop\n");
+	rpglog(RPGDEBUGLEVEL+100 , "libnet:read_packet start\n");
 	if (timeout <= 0)timeout= 1000;
 
 	if (requestlen > bufsizei){
@@ -971,6 +1086,8 @@ static int read_packet(int FD	 , int requestlen , int *read_len , char * buf , i
 		return (stat);
 	}
 	/*ɤ߹ߤγ*/
+	rpglog(RPGDEBUGLEVEL+100 , "libnet:read_packet:start read[%d] requestlen[%d] *read_len[%d]\n", 
+			requestlen - *read_len,requestlen,*read_len);
 	ret = read (FD , &buf[*read_len] , requestlen - *read_len);
 	if (ret< 0){
 		stat = LIBNETBASEERR - errno;
@@ -1036,22 +1153,28 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 	int new_fd;
 	int len;
 	int cnt2;
+	rpglog(RPGDEBUGLEVEL+100 , "libnet:RPG_Poll\n");
 
 	*length = 0;
 	bzero (command,COMMANDLEN);
 	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
-		if (rpg_FDS[cnt] != 0){
+		if (rpg_FDS[cnt] != 0 || rpg_FDTYPE[cnt] ==POLLTYPE_RAW ){
 			s_Poll[i].events = POLLIN;
 			s_Poll[i].fd = rpg_FDS[cnt];
 			i++;
+			rpglog(RPGDEBUGLEVEL2 , "libnet:set fd [%d] type [%d]\n",rpg_FDS[cnt],rpg_FDTYPE[cnt]);
+		}else{
+			rpglog(RPGDEBUGLEVEL2 , "libnet:not set  fd [%d] type [%d]\n",rpg_FDS[cnt],rpg_FDTYPE[cnt]);
 		}
 	}
+	rpglog(RPGDEBUGLEVEL+100 , "libnet:RPG_Poll:poll\n");
 	/*٥Ȥμ*/
 	nStat = poll( s_Poll, i , timeout );
 	if ( nStat > 0 )	/* ٥Ȥȯ */
 	{
 		 /* ٥Ȥǥѥå */
 		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt++){
+			cnt2 = test_fd(s_Poll[cnt].fd);
 			if (s_Poll[cnt].revents){
 				if (s_Poll[cnt].revents > POLLERR){
 					rpg_closefd (s_Poll[cnt].fd);
@@ -1059,9 +1182,9 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 				}
 				*resfd = s_Poll[cnt].fd;
 				/*UNIXåȤ³׵*/
-				if ((s_Poll[cnt].fd ==	servfdU) && (servfdU != 0)){
+				if (POLLTYPE_USERV == rpg_FDTYPE[cnt2] || POLLTYPE_URAWSERV == rpg_FDTYPE[cnt2]){
 					struct sockaddr_un	  caddr;
-					rpglog(RPGDEBUGLEVEL , "libnet:got connect ion request \n");
+					rpglog(RPGDEBUGLEVEL , "libnet:RPG_Poll:got connection request \n");
 					len = sizeof(caddr);
 					new_fd = getfdnewentry();
 					if ((rpg_FDS[new_fd] = accept(s_Poll[cnt].fd ,	
@@ -1080,10 +1203,16 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 					/*׵ƥץȤ饳ͥȤȤ֤*/
 					rpglog (RPGDEBUGLEVEL , "libnet:return that have  connect [%d]-[%d] \n"
 					, new_fd , rpg_FDS[new_fd] );
+					if (POLLTYPE_URAWSERV == rpg_FDTYPE[cnt2]){
+						rpg_FDTYPE[new_fd]=POLLTYPE_RAW;
+						memcpy (command , LIBNETRAWCONNECTREQ , COMMANDLEN);
+						return LIBNETCONNECTREQUESTRAWU;
+					}
+					rpg_FDTYPE[new_fd]=POLLTYPE_LAUREL;
 					return (LIBNETCONNECTREQUESTU);
 				}
 				/*³׵ץץȤơƣĤϿ*/
-				if ((s_Poll[cnt].fd ==	servfdT) && (servfdT != 0)){
+				if (POLLTYPE_TSERV == rpg_FDTYPE[cnt2] || POLLTYPE_TRAWSERV == rpg_FDTYPE[cnt2]){
 					struct sockaddr_in	  caddr;
 					rpglog (RPGDEBUGLEVEL , "libnet:conection request\n");
 					len = sizeof(caddr);
@@ -1121,11 +1250,17 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 					memcpy (command , LIBNETCONNECTREQ , COMMANDLEN);
 					rpglog (RPGDEBUGLEVEL , "libnet:return connect if it was sucsess[%d]-[%d] from[%s]\n"
 					, new_fd , rpg_FDS[new_fd],inet_ntoa(caddr.sin_addr));
+					if (POLLTYPE_TRAWSERV == rpg_FDTYPE[cnt2]){
+						rpg_FDTYPE[new_fd]=POLLTYPE_RAW;
+						memcpy (command , LIBNETRAWCONNECTREQ , COMMANDLEN);
+						return LIBNETCONNECTREQUESTRAWT;
+					}
+					rpg_FDTYPE[new_fd]=POLLTYPE_LAUREL;
 					return LIBNETCONNECTREQUESTT;
 				}
 				cnt2 = test_fd(s_Poll[cnt].fd);
-				/*rpg_IDSäңףƣļȤƽ*/
-				if (rpg_IDS[cnt2]==-1){
+				/*POLLTYPE_RAW==rpg_FDTYPEäңףƣļȤƽ*/
+				if (POLLTYPE_RAW==rpg_FDTYPE[cnt2]){
 					/*RAW data äϤΤޤ޼ʤǤ*/
 					*length=0;
 					*resfd=s_Poll[cnt].fd;
@@ -1134,6 +1269,7 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 						*resfd,*length);
 					return LIBNETRAWRESEVE;
 				}
+				rpglog(RPGDEBUGLEVEL+100 , "libnet:read data head[%d]\n",sizeof (libnet_send_st));
 				/*ǡξ硢إåᤷƽ*/
 				ret = read (s_Poll[cnt].fd , &packet_head , sizeof (libnet_send_st));
 				if (ret < 0){
@@ -1179,6 +1315,7 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 					/*ͤλϥ顼֤*/
 					return LIBNETNOREQLEN;
 				}
+				rpglog(RPGDEBUGLEVEL+100 , "libnet:read data [%d]\n",requestlen);
 				/*μν򤹤롣*/
 				if (cnt2 < 0)return LIBNETERROTHER;
 				
@@ -1442,7 +1579,7 @@ int rpg_loginit( char * pname  , char * sockname){
 			}
 			if (pname != NULL){
 				memset (rpgPname , 0 , 20);
-				memcpy (rpgPname , pname , 10);
+				memcpy (rpgPname , pname , 15);
 			}
 			logconnect = 1;
 		}else{
@@ -1458,11 +1595,12 @@ int rpg_loginit( char * pname  , char * sockname){
 /*顼ν򤹤*/
 int loginit( char * pname ){
 	int ret = 0;
+	if (rpg_dbglvl(0) > 2000)printf( "libnet:loginit:initial log server[%s]\n",pname);
 	if (rpg_logsockname(NULL)!=""){
 		if ( logconnect == 0){/*⤷Ф³Ƥʤä³򤹤*/
 			if ( rpglogfd == 0){
 				if ( (ret = rpg_usoc_client_open(rpg_logsockname(NULL))) < 0){
-					rpglog (RPGDEBUGLEVEL , "libnet:cant connect log server\n");
+					printf ("libnet:cant connect log server\n");
 					return ret;
 				}
 				rpglogfd = ret;
@@ -1473,7 +1611,7 @@ int loginit( char * pname ){
 			}
 			logconnect = 1;
 		}else{
-			rpglog(RPGDEBUGLEVEL , "libnet:log server already connected \n");
+			printf("libnet:log server already connected \n");
 			/*ФΣƣĤ֤*/
 			return rpglogfd;
 		}
@@ -1500,7 +1638,7 @@ char errerbufffer[LOGBUFFSIZE];
 char tmpbuf[LOGBUFFSIZE];
 char filename[sizeof(int) +LOGBUFFSIZE];
 va_list ap;
-
+	if (rpg_dbglvl(0) > 3000)printf ("libnet:rpglog:start   rpglogfd[%d]\n",rpglogfd);
 	/*ХåեΥꥢ򤹤*/
 	memset (errerbufffer ,(int) NULL , LOGBUFFSIZE);
 	memset (tmpbuf , (int)NULL , LOGBUFFSIZE);
@@ -1520,6 +1658,7 @@ va_list ap;
 	va_end(ap);
 	/*ФȤ³򤷤ƤʤȤϡ*/
 	if (logconnect == 0){
+		if (rpg_dbglvl(0) > 3000)printf ("not connect to log server\n");
 		if (rpg_dbglvl(0) > DBlev){
 			if (rpg_dbglvl(0)%2)printf (errerbufffer);
 			if (rpg_dbglvl(0)%2)printf (tmpbuf);
@@ -1529,6 +1668,7 @@ va_list ap;
 	}
 	/*ƣĤϿƤ뤫ǧ*/
 	if (test_fd(rpglogfd) < 0){
+		printf ("log server FD not registered\n");
 		if ((ret = loginit (rpgPname)) < 0){
 			logconnect = 0 ;
 			rpg_gamelog(DBlev , tmpbuf);
@@ -1539,7 +1679,7 @@ va_list ap;
 	n=1;
 	/*ФФ򤹤*/
 	*((int *)filename)=htonl(DBlev);
-	if (rpglogfd > 0){
+	if (rpglogfd > 0&& ((rpg_dbglvl(0) > DBlev) || DBlev < 1000)){
 #if 0
 		strncpy (&filename[sizeof (int)] , errerbufffer , strlen (errerbufffer));
 		strcat (&filename[sizeof (int)] , rpgPname);	/*襻ơۡˤʤǽ*/
@@ -1551,20 +1691,24 @@ va_list ap;
 		}
 		ret = rpg_send(rpglogfd , RPGLOGREQ , filename	, sizeof(int)+strlen (&filename[sizeof (int)]));
 		if (ret <0){
+			printf ("rpglog:rpg_send:error [%d] %s\n",ret,rpgPname);
 			rpg_closefd(rpglogfd);
 			rpglogfd=0;
 			logconnect=0;
 			loginit( rpgPname);
 		}
-	}else{
+	}else if (rpglogfd < 0){
+		printf ("rpglog: rpglogfd[%d] DBlev[%d]\n");
 		rpg_gamelog(DBlev , tmpbuf);
 		logconnect=0;
 		loginit( rpgPname);
 		return LIBNETNOFD;
 	}
 	/*ǥХå٥뤬λɸ२顼ˤФ*/
-	if (rpg_dbglvl(0)%2)write (2, errerbufffer,strlen (errerbufffer));
-	if (rpg_dbglvl(0)%2)write (2,tmpbuf,strlen (tmpbuf));
+	if (rpg_dbglvl(0) > DBlev){
+		if (rpg_dbglvl(0)%2)write (2, errerbufffer,strlen (errerbufffer));
+		if (rpg_dbglvl(0)%2)write (2,tmpbuf,strlen (tmpbuf));
+	}
 	return (ret);
 }
 
@@ -1697,6 +1841,10 @@ int ret;
 	if (shmNO >= LIBNETSHMIDMAX){
 		return (char *)LIBNETSHMIDerror;
 	}
+	if (RPG_shm==NULL){
+		return (char *)LIBNETSHMNOINIT;
+
+	}
 	if (RPG_shm[shmNO].shm_id != (int)NULL && RPG_shm[shmNO].size != (int)NULL){
 		return (char *)LIBNETSHMaredyattach;
 	}
@@ -1723,6 +1871,9 @@ int ret;
 void PRG_shmctl_destroy(){
 	int cnt;
 	
+	if (RPG_shm==NULL){
+		return ;
+	}
 	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++){
 		if (RPG_shm[cnt].shm_id != (int)NULL && RPG_shm[cnt].size != (int)NULL){
 			if (RPG_shm[cnt].adder !=NULL){
@@ -1734,12 +1885,16 @@ void PRG_shmctl_destroy(){
 			}
 		}
 	}
+	FREE (RPG_shm);
 }
 
 /*ͭΥå򤹤*/
 
 char * RPG_shm_attach(int shmNO){
 char * adder;
+	if (RPG_shm==NULL){
+		return (char *)LIBNETSHMNOINIT;
+	}
 	if (RPG_shm[shmNO].shm_id== (int)NULL){
 		
 		return (char *)LIBNETSHMNOSHMID;
@@ -1770,6 +1925,9 @@ static RPG_shmunit * ctladder= NULL;
 	if ((void *)ctladder <= NULL){
 		return	((RPG_shmunit *)LIBNETSHMMALOCCERR);
 	}
+	if (RPG_shm==NULL){
+		return (RPG_shmunit *)LIBNETSHMNOINIT;
+	}
 	memcpy (ctladder , RPG_shm , LIBNETSHMIDMAX * sizeof(RPG_shmunit) );
 	
 	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++){
@@ -1804,22 +1962,13 @@ void  RPG_shm_all_detach(){
 /*ƤΥǥå*/
 
 void  RPG_shm_detach(){
-	int cnt;
 	RPG_shmunit * ctladder;
 	rpglog (RPGDEBUGLEVEL , "libnet:RPG_shm_detach start\n");
 	ctladder = RPG_shm_init((int)NULL);
 	if (ctladder == NULL){
 		return;
 	}
-	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++){
-		rpglog (RPGDEBUGLEVEL , "libnet:RPG_shm_detach[%d][%d]\n" , cnt , ctladder[cnt].adder);
-		if (ctladder[cnt].adder != NULL){
-			shmdt(ctladder[cnt].adder);
-			ctladder[cnt].adder = NULL;
-			ctladder[cnt].shm_id = (int)NULL;
-			ctladder[cnt].size = (int)NULL;
-		}
-	}
+	PRG_shmctl_destroy();
 }
 /**********************************************************
 ͭmmapؤ
@@ -2067,9 +2216,13 @@ errtable errmessage[MAXERRNOM] = {
 		/*꤬˥ץ󤵤Ƥ*/
 
 /*н*/
-	"The memory was aredy open on same ID \n Dont warry you dont have to do any thing.i\nthis is worninng." } 	,
+	"The memory was alredy open on same ID \n Dont warry you dont have to do any thing.i\nthis is worninng." } 	,
 	/*Ǥ˶꤬ͭץ󤵤Ƥ*/
 
+{  20002,"poll fd alredy open",""},		/*ǤˣƣĤоݤȤϿƤ*/
+{  10012,"TCP RAWSERVER connection receive",""},		/*TCP RAWSERVER ³׵*/
+{  10011,"UNIX SOC RAWSERVER connection receive",""},		/*UNIX SOC RAWSERVER³׵*/
+{  10010,"receive from raw fd",""},		/*RAWFDȯ*/
 {  10009,"Got connection request",""},		/*³׵*/
 {  10008,"Got connection request",""},		/*³׵*/
 {  10007,"Open FD",""},		/*ƣĤ򥪡ץ󤷤*/
@@ -2078,10 +2231,10 @@ errtable errmessage[MAXERRNOM] = {
 {  10004,"Lost part of data",""},		/*ʬŪ˥ǡ򼺤ä*/
 {  10003,"ID isn't continued",""},		/*ɣĤϢ³Ƥʤ*/
 {  10002,"Data buffer shorter than input data",""},		/*ǡХåեξ꾮*/
-{  10001,"",""},		/**/
+{  10001,"success",""},		/**/
 { -10001,"FD was not enterd",""},		/*ƣĤäƤʤ*/
 { -10002,"It was no FD entry",""},		/*ƣĤΥȥ꤬ʤ*/
-{ -10003,"",""},		/**/
+{ -10003,"send error",""},		/**/
 { -10004,"header cant read compleate",""},		/*إåɤ߹ޤʤä*/
 { -10005,"cant alloc memory",""},		/*γݼ*/
 { -10006,"falt make socket",""},		/*åȤκ顼*/
@@ -2112,6 +2265,7 @@ errtable errmessage[MAXERRNOM] = {
 { -20005,"no memory ID entry",""},		/*ͭɣĤϿƤޤ*/
 { -20006,"falt malloc",""},		/*Υޥå˼Ԥޤ*/
 { -20007,"no memory ID entry",""},		/*ͭɣĤϿƤޤ*/
+{ -20008,"shared memory not initialized",""},		/*ͭꥳȥ뤬Ƥʤ*/
 { -30001,"",""},
 {0,"",""}
 };
