@@ -95,91 +95,103 @@
 
 #define LIBNET
 #define LIBNET
-#define LIBNETOREDYOPEN			20001		/*Ǥ˶꤬ͭץ󤵤Ƥ*/	
-#define LIBNETFDAREDYSET		20002		/*ǤˣƣĤоݤȤϿƤ*/
-#define LIBNETRAWRESEVE			10010		/*RAWFDȯ*/
-#define LIBNETCONNECTREQUESTT	10009		/*³׵*/
-#define LIBNETCONNECTREQUESTU	10008		/*³׵*/
-#define LIBNETFDOPEN			10007		/*ƣĤ򥪡ץ󤷤*/
-#define LIBNETNOTOPENLOG		10006		/*ץ󤵤Ƥʤ*/
-#define LIBNETWTIMEUP			10005		/*׵ᤵ줿٥ȤΤɤ줫˥ʥ뤬ȯ*/
-#define LIBNETDATAPARTIALLOST	10004		/*ʬŪ˥ǡ򼺤ä*/
-#define LIBNETNOIDCONTINU		10003		/*ɣĤϢ³Ƥʤ*/
-#define LIBNETLOSTDATA			10002		/*ǡХåեξ꾮*/
-#define LIBNETOK				10001		/**/
-#define LIBNETNOFD				-10001		/*ƣĤäƤʤ*/
-#define LIBNETNOFDENT			-10002		/*ƣĤΥȥ꤬ʤ*/
-#define LIBNETSENDERR			-10003		/**/
-#define LIBNETHEADSENDERR		-10004		/*إåɤ߹ޤʤä*/
-#define LIBNETMALOCCERR			-10005		/*γݼ*/
-#define LIBNETMAKESOCERR		-10006		/*åȤκ顼*/
-#define LIBNETGETPORTERR		-10007		/*ݡֹμ顼*/
-#define LIBNETSETHOSTNEMEERR	-10008		/*ۥȥ͡Υåȼ*/
-#define LIBNETCONECTERR			-10009		/*ͥȥ顼*/
-#define LIBNETSIGSETERR			-10010		/*ʥΥåȥ顼*/
-#define LIBNETPOLLERR			-10011		/* Error condition */
-#define LIBNETPOLLHUP			-10012		/* Hung up */
-#define LIBNETPOLLNVAL			-10013		/* Invalid request: fd not open */
-#define LIBNETEFAULT			-10015		/*Ȥƻꤷ󤬡ƤӽФץΥɥ쥹֤˴ޤޤƤʤ*/
-#define LIBNETERR				-10014		/*¾Υ顼*/
-#define LIBNETHEADBREAK			-10015		/*إåʬƤ*/
-#define LIBNETREADBREAK			-10016		/*ɤ߹ߤΥʥˤ*/
-#define LIBNETNOREQLEN			-10017		/*׵Ĺޤϥޥʥ*/
-#define LIBNETOTHERSIG			-10018		/* ͽʤʥˤ*/
-#define LIBNETOREDYSOPEN		-10019		/*Ǥ˥Хץ󤷤Ƥ*/
-#define LIBNETLENGTHZERO		-10020		/*Ĺξ*/
-#define LIBNETCONNECTIONERROR	-10021		/*³μ*/
-#define LIBNETCONNECTIONDATAERR	-10022		/*³³ʸǤʤä*/
-#define LIBNETCATCREATFD		-10023		/*ƥ¤ǤʾƣĤ򥪡ץǤʤ*/
-#define LIBNETREADLENLESS		-10024		/*ɤ߹Ĺ*/
-#define LIBNETSOCBINDERR		-10025		/*åȤΥХɥ顼*/
-#define LIBNETLISSTENERR		-10026		/*listen򥽥åȤФȯԼ*/
-#define LIBNET
-#define LIBNETERROTHER			-10100		/*¾ͽʤ顼*/
-#define LIBNETSHMaredyattach	-20001		/*Ǥ˥ץ󤵤줿ɣ*/
-#define LIBNETSHMNOIDentry		-20002		/*Ǥ˥ץ󤵤줿ɣ*/
-#define LIBNETSHMATTACHERR		-20003		/*ͭꥢå顼*/
-#define LIBNETSHMGETERR			-20004		/*ͭꥲåȥ顼*/
-#define LIBNETSHMIDerror		-20005		/*ͭɣĤϿƤޤ*/
-#define LIBNETSHMMALOCCERR		-20006		/*Υޥå˼Ԥޤ*/
-#define LIBNETSHMNOSHMID		-20007		/*ͭɣĤϿƤޤ*/
+#define LIBNETOREDYOPEN				20001		/*Ǥ˶꤬ͭץ󤵤Ƥ*/	
+#define LIBNETFDAREDYSET			20002		/*ǤˣƣĤоݤȤϿƤ*/
+#define LIBNETCONNECTREQUESTRAWT	10012		/*TCP RAWSERVER ³׵*/
+#define LIBNETCONNECTREQUESTRAWU	10011		/*UNIX SOC RAWSERVER³׵*/
+#define LIBNETRAWRESEVE				10010		/*RAWFDȯ*/
+#define LIBNETCONNECTREQUESTT		10009		/*TCP ³׵*/
+#define LIBNETCONNECTREQUESTU		10008		/*UNIX SOCK ³׵*/
+#define LIBNETFDOPEN				10007		/*ƣĤ򥪡ץ󤷤*/
+#define LIBNETNOTOPENLOG			10006		/*ץ󤵤Ƥʤ*/
+#define LIBNETWTIMEUP				10005		/*׵ᤵ줿٥ȤΤɤ줫˥ʥ뤬ȯ*/
+#define LIBNETDATAPARTIALLOST		10004		/*ʬŪ˥ǡ򼺤ä*/
+#define LIBNETNOIDCONTINU			10003		/*ɣĤϢ³Ƥʤ*/
+#define LIBNETLOSTDATA				10002		/*ǡХåեξ꾮*/
+#define LIBNETOK					10001		/**/
+#define LIBNETNOFD					-10001		/*ƣĤäƤʤ*/
+#define LIBNETNOFDENT				-10002		/*ƣĤΥȥ꤬ʤ*/
+#define LIBNETSENDERR				-10003		/**/
+#define LIBNETHEADSENDERR			-10004		/*إåɤ߹ޤʤä*/
+#define LIBNETMALOCCERR				-10005		/*γݼ*/
+#define LIBNETMAKESOCERR			-10006		/*åȤκ顼*/
+#define LIBNETGETPORTERR			-10007		/*ݡֹμ顼*/
+#define LIBNETSETHOSTNEMEERR		-10008		/*ۥȥ͡Υåȼ*/
+#define LIBNETCONECTERR				-10009		/*ͥȥ顼*/
+#define LIBNETSIGSETERR				-10010		/*ʥΥåȥ顼*/
+#define LIBNETPOLLERR				-10011		/* Error condition */
+#define LIBNETPOLLHUP				-10012		/* Hung up */
+#define LIBNETPOLLNVAL				-10013		/* Invalid request: fd not open */
+#define LIBNETEFAULT				-10015		/*Ȥƻꤷ󤬡ƤӽФץΥɥ쥹֤˴ޤޤƤʤ*/
+#define LIBNETERR					-10014		/*¾Υ顼*/
+#define LIBNETHEADBREAK				-10015		/*إåʬƤ*/
+#define LIBNETREADBREAK				-10016		/*ɤ߹ߤΥʥˤ*/
+#define LIBNETNOREQLEN				-10017		/*׵Ĺޤϥޥʥ*/
+#define LIBNETOTHERSIG				-10018		/* ͽʤʥˤ*/
+#define LIBNETOREDYSOPEN			-10019		/*Ǥ˥Хץ󤷤Ƥ*/
+#define LIBNETLENGTHZERO			-10020		/*Ĺξ*/
+#define LIBNETCONNECTIONERROR		-10021		/*³μ*/
+#define LIBNETCONNECTIONDATAERR		-10022		/*³³ʸǤʤä*/
+#define LIBNETCATCREATFD			-10023		/*ƥ¤ǤʾƣĤ򥪡ץǤʤ*/
+#define LIBNETREADLENLESS			-10024		/*ɤ߹Ĺ*/
+#define LIBNETSOCBINDERR			-10025		/*åȤΥХɥ顼*/
+#define LIBNETLISSTENERR			-10026		/*listen򥽥åȤФȯԼ*/
+#define LIBNET	
+#define LIBNETERROTHER				-10100		/*¾ͽʤ顼*/
+#define LIBNETSHMaredyattach		-20001		/*Ǥ˥ץ󤵤줿ɣ*/
+#define LIBNETSHMNOIDentry			-20002		/*Ǥ˥ץ󤵤줿ɣ*/
+#define LIBNETSHMATTACHERR			-20003		/*ͭꥢå顼*/
+#define LIBNETSHMGETERR				-20004		/*ͭꥲåȥ顼*/
+#define LIBNETSHMIDerror			-20005		/*ͭɣĤϿƤޤ*/
+#define LIBNETSHMMALOCCERR			-20006		/*Υޥå˼Ԥޤ*/
+#define LIBNETSHMNOSHMID			-20007		/*ͭɣĤϿƤޤ*/
+#define LIBNETSHMNOINIT				-20008		/*ͭꥳȥ뤬Ƥʤ*/
 #define LIBNETSHM
 #define LIBNETSHM
-#define LIBNETSHM
-#define RPGMALLOCERROR			-30001
+#define RPGMALLOCERROR				-30001
 
-#define RPGLOGREQ			"RPGLOGREQ_"			/*׵*/
+#define RPGLOGREQ					"RPGLOGREQ_"			/*׵*/
 
-#define LIBNETBASEERR			-500
+#define LIBNETBASEERR				-500
 
-#define LIBNETSHMIDMAX			20			/*ͭ票ȥ꡼*/
+#define LIBNETSHMIDMAX				20			/*ͭ票ȥ꡼*/
 
-#define LIBNETPOLLrerycount		5
+#define LIBNETPOLLrerycount			5
 
-/*#define LIBNETLOGSOCNAME		"log_soc"*/
-#define LIBNETLOGSOCNAME		""
-#define LIBNETTIMEUP			100
+/*#define LIBNETLOGSOCNAME			"log_soc"*/
+#define LIBNETLOGSOCNAME			""
+#define LIBNETTIMEUP				100
+
+#define COMMANDLEN					10
+#define LOGLINEMAX					10000			/*κ饤*/
+#define LOGFILENAME					"Laurier.log"
+#define LOGBUFFSIZE					256
+#define MAXERRNOM					100
+#define LIBNET_UDP					3
+
+/*FD type*/
+#define POLLTYPE_LAUREL				0	/*laurelפ̾Σƣġͥλޤϣգģ*/
+#define POLLTYPE_RAW				1	/*ףƣ			ͥλޤϣգģФޤϳϿ*/
+#define POLLTYPE_TSERV				2   /*laurelפΥУƣ*/
+#define POLLTYPE_USERV				3   /*գΣɣإåȤlaurelפΥУƣ*/
+#define POLLTYPE_TRAWSERV			4   /*rawפΥУƣ*/
+#define POLLTYPE_URAWSERV			5   /*գΣɣإåȤrawפΥУƣ*/
 
-#define COMMANDLEN			10
-#define LOGLINEMAX			10000			/*κ饤*/
-#define LOGFILENAME			"Laurier.log"
-#define LOGBUFFSIZE			256
-#define MAXERRNOM			100
-#define LIBNET_UDP			3
 #ifndef MAXPATHLEN
-#	define MAXPATHLEN			512
+#	define MAXPATHLEN				512
 #endif
 /**************************************************************************/
 /*Υإåι¤
 ǥѥåȤĤʤäڤ줿ꤷˤб롣
 Υإåκǽ*/
 
-#define LIBNETPHEADER			"NRPGHEAD"	/*ѥåȥإå*/
-#define LIBNETASSEPT			"LIBNETASSEPT"	 
-#define LIBNETCONNECTREQ		"CONNECTREQ"	/*³׵*/
-#define POLLTIMEUP          	"POLLTIMEUP"            /**/
+#define LIBNETPHEADER				"NRPGHEAD"	/*ѥåȥإå*/
+#define LIBNETASSEPT				"LIBNETASSEPT"	 
+#define LIBNETCONNECTREQ			"CONNECTREQ"	/*³׵*/
+#define LIBNETRAWCONNECTREQ			"CNCTRAWREQ"	/*³׵*/
+#define POLLTIMEUP          		"POLLTIMEUP"            /**/
 #define LIBNETRAWDATARECEVE			"RAWDATAREC"
-#define LIBNETWTIMEUP2			"LIBNETIMUP"
+#define LIBNETWTIMEUP2				"LIBNETIMUP"
 /*		ʸ󤬤뤳,
 		length
 		ID
@@ -234,66 +246,6 @@ typedef struct{
 
 **********************************************************/
 
-/**************************************************************************
-	
-			ѥ»Хåؿ
-
-	:()
-		ѥפȤʤɤ˸ƤӽФ
-		Хåؿ
-		äˤʤˤ⤷ʤǵ
-
-	:
-		ʤ
-
-	:
-		ʤ
-*/
-
-			void brakepipe(int data);
-
-
-/**************************************************************************
-
-			FD¸߳ǧؿ
-
-	()
-		ǡǡ١ˤΣƣĤäƤ뤫ǧ
-		ơΥȥ꡼ֹ֤
-
-	:
-		int fd 		I	Ĵ٤ƣ
-
-	:
-		0<
-			䥨ȥ꡼ֹ
-		0>
-			顼
-				LIBNETNOFDȥ꡼¸ߤʤ
-*/
-
-static	int test_fd(int fd );
-
-
-/*************************************************************************/
-
-/*			FDϿؿ
-
-	:()
-		FDǡ١˿ȥä
-		Υȥֹ֤
-		Ρȥξˣ򥪡ץ󤷤
-		ƣĤ롣
-
-	
-		ʤ
-
-	͡
-		Ƥƣĥǡ١ȥ
-*/
-
-static int getfdnewentry(void);
-
 
 
 /**************************************************************************/
@@ -330,22 +282,6 @@ void	rpg_closefd(int FD);
 
 void rpg_fd_allclose(void);
 
-/**************************************************************************/
-
-/*			ǡ١ؿ
-	
-		ǡ١
-		ǽΰƤӽФ줿˽򤷤Ƥʹ
-		˽򤹤Ȥ̤δؿƤӽФ
-
-	
-		ʤ
-
-*/
-
-static int clear_fddatabase(void);
-
-
 
 /**************************************************************************/
 /*			ҥץ˴ƤӽФؿ
@@ -365,6 +301,25 @@ void  rpg_childdeth(int data);
 
 void rpg_endproc(int data);
 
+/**************************************************************************/
+/*	
+		ңסƣĤϿ
+
+   	:
+		int FD				I
+		Ͽƣ
+	͡
+		LIBNETOK			
+		LIBNETFDAREDYSET	ǤϿƤƣ
+
+	ǤϿƤƣĤ
+	Ȥäƺ줿̿ƣĤϿ뤳ȤǤʤ
+	Ǻä̤Υ饤֥ǺäƣĤϿΤ˻Ѥ롣
+ 
+ */
+
+int rpg_addRawPollFD(int FD);
+
 
 /**************************************************************************/
 /*	
@@ -392,6 +347,51 @@ void rpg_endproc(int data);
 
 int rpg_serveropen (int type ,	char * service_name );
 
+/**************************************************************************/
+/*	
+		RAWѥСץ󤹤
+		ԣãФȣգΣɣإåȤ
+
+	:
+		int type 				I
+			˥ååȤΤȤϡ
+			type = AF_UNIX
+			TCPλ
+			TYPE = AF_INET
+			UDPλ
+			TYPE= LIBNET_UDP
+			Ȥ롣
+
+		char * service_name		I
+			ΤȤservice_nameϣԣãФλϥӥ̾
+			˥ååȤλϥå̾ΤϤ
+
+	:
+			ƣĤ֤
+			顼ξ֤ͤ
+
+	ΥץȥѤʤ³׵դ
+	СåȤ򥪡ץ󤹤
+
+	δؿȤäƥСץ󤹤ȡrpg_poll³׵ϼưǼ
+		#define LIBNETCONNECTREQUESTRAWT    10012       TCP RAWSERVER ³׵
+		#define LIBNETCONNECTREQUESTRAWU    10011       UNIX SOC RAWSERVER³׵
+		֤
+		ޥϤäƤ
+		#define LIBNETRAWCONNECTREQ         "CNCTRAWREQ"   
+		
+		ưŪ³׵ƤޤȤ
+
+		ʬǼȤˤϥСץǹԤ
+		ΣƣĤңףϿ򤷤ƻѤ
+			int rpg_addRawPollFD(int FD)Ѥ롣
+	ʾ	rpg_poll()
+
+ 
+ */
+
+int rpg_RawServerOpen(int type ,    char * service_name );
+
 
 /**************************************************************************/
 
@@ -513,7 +513,7 @@ int rpg_eventloop( char * data , int * leng	 , struct  timeval timeout );
 
 	֤͡
 */
-static int read_packet (int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout);
+/*static int read_packet (int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout);*/
 
 
 
@@ -704,6 +704,11 @@ int rpg_dbglvl(int dbglvl);
 */
 int rpg_gamelog(int dbglvl ,char * tmpbuf );
 
+/* С
+ *  * 饤ȤΡRPGLOGREQ򤦤
+ *   * ΥǡȤϤ
+ *    * ǥХå٥򥻥åȤƥ󥰤*/
+int rpg_receveLogwrite(char * readbuf);
 
 /**********************************************************/
 /**********************************************************
